/*
* Licensed to the Apache Software Foundation (ASF) under one
* or more contributor license agreements.  See the NOTICE file
* distributed with this work for additional information
* regarding copyright ownership.  The ASF licenses this file
* to you under the Apache License, Version 2.0 (the
* "License"); you may not use this file except in compliance
* with the License.  You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing,
* software distributed under the License is distributed on an
* "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
* KIND, either express or implied.  See the License for the
* specific language governing permissions and limitations
* under the License.
*/
var game;
var app = {
   // Application Constructor
   initialize: function() {
      this.bindEvents();
   },
   // Bind Event Listeners
   //
   // Bind any events that are required on startup. Common events are:
   // 'load', 'deviceready', 'offline', and 'online'.
   bindEvents: function() {
      document.addEventListener('deviceready', this.bootstrapGame, false);
      document.addEventListener('pause', this.saveGameState, false);
      document.addEventListener('resume', this.setRetryText, false);
   },

   // Update DOM on a Received Event
   bootstrapGame: function() {
      console.log('Bootstrapping game now');
      game = gameBrain();   
      var arrowMap = {
         37: 'swipeleft',
         38: 'swipeup',
         39: 'swiperight',
         40: 'swipedown'
      };
      var board = $('#board');
      var hammer = new Hammer(board[0]);
      hammer.get('swipe').set({
         direction: Hammer.DIRECTION_ALL,
         velocity: 0.25
      });

      hammer.on('swipeleft swiperight swipeup swipedown', 
         _.throttle(function(e){
            $(board).trigger(e.type);
         }, 300));

      // hammer.on('swipeleft swiperight swipeup swipedown', function(e){
      //   $(board).trigger(e.type);
      // });

      $(board).on('touchmove', function(e){
         e.preventDefault();
      });

      if (googleplaygame && game.shouldAutoLogin()) {
         googleplaygame.auth(function () {
            // no-op when successful
            console.log('auto-login...');
         }, function () {
            googleplaygame.userCancelledAutoSignIn(function (result) {
               if (result.hasCancelled) {
                  console.log('user cancelled the auto sign in!!');
                  game.supressAutoGoogleLogin();
               }
            });
         });
      }

      game.init();
   },

   saveGameState: function () {
      console.log('pause/kill detected: saving game state');
      game.saveGameState();      
   },

   setRetryText: function () {
      // Whenever we save, we override the save state, so assume they can't
      // retry. Only exception is when we hit the 'retry' button (in which case)
      // the text is handled later.      
      // Maybe in the future, save 'retry' state into diff variable?
      $('#retry').html('retry');
   }
};
